/* Translators (2009 onwards):
 *  - Klenje
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["fur"]
 * Dictionary for Furlan.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["fur"] = OpenLayers.Util.applyDefaults({

    'Permalink': "Leam Permanent",

    'Overlays': "Livei parsore",

    'Base Layer': "Livel di base",

    'browserNotSupported': "Il to sgarfadôr nol supuarte la renderizazion vetoriâl. Al moment a son supuartâts:\n${renderers}",

    'Scale = 1 : ${scaleDenom}': "Scjale = 1 : ${scaleDenom}",

    'W': "O",

    'E': "E",

    'N': "N",

    'S': "S"

});
