/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("en", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div>Zoom Level: {zoom}</div><div>Scale: 1:{scale}</div>",
        loadConfigErrorText: "Trouble reading saved configuration: <br />",
        loadConfigErrorDefaultText: "Server Error.",
        xhrTroubleText: "Communication Trouble: Status ",
        layersText: "Layers",
        titleText: "Title",
        zoomLevelText: "Zoom level",
        bookmarkText: "Bookmark URL",
        permakinkText: 'Permalink',
        appInfoText: "GeoExplorer",
        aboutText: "About GeoExplorer",
        mapInfoText: "Map info",
        descriptionText: "Description",
        contactText: "Contact",
        aboutThisMapText: "About this map"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Map",
        tableText: "Table",
        queryText: "Query",
        exportMapText: "Publish map",
        saveMapText: "Save map",
        toolsTitle: "Choose tools to include in the toolbar:",
        previewText: "Preview",
        backText: "Back",
        nextText: "Next",
        loginText: "Login",
        loginErrorText: "Invalid username or password.",
        userFieldText: "User",
        passwordFieldText: "Password",
        logoutConfirmTitle: "Warning",
        logoutConfirmMessage: "Logging out will undo any unsaved changes, remove any layers you may have added, and reset the map composition. Do you want to save your composition first?"
    }
});
