/* Translators (2009 onwards):
 *  - វ័ណថារិទ្ធ
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["km"]
 * Dictionary for ភាសាខ្មែរ.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["km"] = OpenLayers.Util.applyDefaults({

    'Permalink': "តំណភ្ជាប់អចិន្ត្រៃយ៍",

    'Base Layer': "ស្រទាប់បាត​",

    'Scale = 1 : ${scaleDenom}': "មាត្រដ្ឋាន = ១ ៖ ${scaleDenom}"

});
