/* Translators (2009 onwards):
 *  - Meno25
 *  - Mutarjem horr
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["ar"]
 * Dictionary for العربية.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["ar"] = OpenLayers.Util.applyDefaults({

    'Permalink': "وصلة دائمة",

    'Base Layer': "الطبقة الاساسية",

    'Scale = 1 : ${scaleDenom}': "النسبة = 1 : ${scaleDenom}",

    'W': "غ",

    'E': "شر",

    'N': "شم",

    'S': "ج"

});
