p {
    text-indent: 0; margin-bottom: 1em;
}

.MGroup {
    font-variant: normal;
    margin: 0.4em 0 0em 10px
}

.MTitle {
    font-variant: normal;
}

.CGroup .CTitle {
    font-variant: normal;
}

.SGroup .SEntry {
    font-variant: normal;
}