/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("fr", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div> Niveau de zoom: {zoom}</div><div Echelle: 1:{scale}</div>",
        loadConfigErrorText: "Difficulté à lire configuration enregistrée: <br>",
        loadConfigErrorDefaultText: "Erreur du serveur.",
        xhrTroubleText: "Trouble de la communication: état",
        layersText: "Layers",
        titleText: "Titre",
        bookmarkText: "Mettre en favori URL",
        permakinkText: "Permalink",
        appInfoText: "GeoExplorer",
        aboutText: "À propos de GeoExplorer",
        mapInfoText: "Info carte",
        descriptionText: "Description",
        contactText: "Contact",
        aboutThisMapText: "À propos de cette carte"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Carte",
        tableText: "Table",
        queryText: "Query",
        exportMapText: "Carte Export",
        saveMapText: "Enregistrez la carte",
        toolsTitle: "Choose tools to include in the toolbar:",
        previewText: "Preview",
        backText: "Back",
        nextText: "Next",
        loginText: "Login",
        loginErrorText: "Invalid username or password.",
        userFieldText: "User",
        passwordFieldText: "Password",
        logoutConfirmTitle: "Warning",
        logoutConfirmMessage: "Logging out will undo any unsaved changes, remove any layers you may have added, and reset the map composition. Do you want to save your composition first?"
    }
});
