var xml = '<?xml version="1.0" encoding="UTF-8"?>' +
'<csw:GetRecordsResponse xmlns:csw="http://www.opengis.net/cat/csw/2.0.2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/cat/csw/2.0.2 http://schemas.opengis.net/csw/2.0.2/CSW-discovery.xsd">' +
'  <csw:SearchStatus timestamp="2012-10-31T15:33:15" />' +
'  <csw:SearchResults numberOfRecordsMatched="155" numberOfRecordsReturned="1" elementSet="full" nextRecord="2">' +
'    <csw:Record xmlns:geonet="http://www.fao.org/geonetwork" xmlns:ows="http://www.opengis.net/ows" xmlns:dc="http://purl.org/dc/elements/1.1/" xmlns:dct="http://purl.org/dc/terms/">' +
'      <dc:identifier>c622b838-ff6c-11e0-ab52-535de64b1b07</dc:identifier>' +
'      <dc:title>Wetlands Study regions</dc:title>' +
'      <dc:subject />' +
'      <dc:subject />' +
'      <dc:format>ShapeFile</dc:format>' +
'      <dct:abstract>This layer is a test to see the possibilities of GeoNode. It represents the Coastal wetlands of Queensland, Australia, divided in 11 study regions.</dct:abstract>' +
'      <dc:language>eng</dc:language>' +
'      <dc:source />' +
'      <dc:format>ShapeFile</dc:format>' +
'      <ows:BoundingBox crs="urn:ogc:def:crs:::WGS 1984">' +
'        <ows:LowerCorner>153.5687595560976 -28.355678693113276</ows:LowerCorner>' +
'        <ows:UpperCorner>137.99467232683276 -10.01045839489004</ows:UpperCorner>' +
'      </ows:BoundingBox>' +
'      <dc:URI protocol="WWW:LINK-1.0-http--link" name="" description="Online link to the \'Wetlands Study regions\' description on GeoNode">http://demo.geonode.org/data/geonode:QLD_DEEDI_Coastal_wetlands_Study_regions</dc:URI>' +
'    </csw:Record>' +
'  </csw:SearchResults>' +
'</csw:GetRecordsResponse>';
