/* Translators (2009 onwards):
 *  - Nike
 *  - Str4nd
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["fi"]
 * Dictionary for Suomi.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["fi"] = OpenLayers.Util.applyDefaults({

    'Permalink': "Ikilinkki",

    'Overlays': "Kerrokset",

    'Base Layer': "Peruskerros",

    'W': "L",

    'E': "I",

    'N': "P",

    'S': "E"

});
