/* Translators (2009 onwards):
 *  - Ævar Arnfjörð Bjarmason
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["is"]
 * Dictionary for Íslenska.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["is"] = OpenLayers.Util.applyDefaults({

    'Permalink': "Varanlegur tengill",

    'Overlays': "Þekjur",

    'Base Layer': "Grunnlag",

    'Scale = 1 : ${scaleDenom}': "Skali = 1 : ${scaleDenom}",

    'methodDeprecated': "Þetta fall hefur verið úrelt og verður fjarlægt í 3.0. Notaðu ${newMethod} í staðin."

});
