/* Translators (2009 onwards):
 *  - DCLXVI
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["bg"]
 * Dictionary for Български.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["bg"] = OpenLayers.Util.applyDefaults({

    'Permalink': "Постоянна препратка",

    'Base Layer': "Основен слой",

    'Scale = 1 : ${scaleDenom}': "Мащаб = 1 : ${scaleDenom}",

    'methodDeprecated': "Този метод е остарял и ще бъде премахват в 3.0. Вместо него използвайте ${newMethod}."

});
