/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("ca", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div>Grau de detall: {zoom}</div><div>Escala: 1:{scale}</div>",
        loadConfigErrorText: "Problemes llegint la configuració desada: <br />",
        loadConfigErrorDefaultText: "Error del servidor.",
        xhrTroubleText: "Problemes de comunicació. Estat ",
        layersText: "Capes",
        titleText: "Títol",
        bookmarkText: "URL del marcador",
        permakinkText: "Enllaç permanent",
        appInfoText: "GeoExplorer",
        aboutText: "Quant a GeoExplorer",
        mapInfoText: "Informació del mapa",
        descriptionText: "Descripció",
        contactText: "Contacte",
        aboutThisMapText: "Quant a aquest mapa"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Mapa",
        tableText: "Table",
        queryText: "Query",
        exportMapText: "Exporta el Mapa",
        saveMapText: "Desa el Mapa",
        toolsTitle: "Escolliu els elements que desitgeu incloure a la barra d'eines:",
        previewText: "Vista prèvia",
        backText: "Anterior",
        nextText: "Següent",
        loginText: "Login",
        loginErrorText: "Invalid username or password.",
        userFieldText: "User",
        passwordFieldText: "Password",
        logoutConfirmTitle: "Warning",
        logoutConfirmMessage: "Logging out will undo any unsaved changes, remove any layers you may have added, and reset the map composition. Do you want to save your composition first?"
    }
});
