/* Translators (2009 onwards):
 *  - Veeven
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["te"]
 * Dictionary for తెలుగు.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["te"] = OpenLayers.Util.applyDefaults({

    'Permalink': "స్థిరలింకు",

    'W': "ప",

    'E': "తూ",

    'N': "ఉ",

    'S': "ద"

});
