/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("es", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div>Nivel de detalle: {zoom}</div><div>Escala: 1:{scale}</div>",
        loadConfigErrorText: "Problemas leyendo la configuración guardada: <br />",
        loadConfigErrorDefaultText: "Error del servidor.",
        xhrTroubleText: "Problemas de comunicación: Estado ",
        layersText: "Capas",
        titleText: "Título",
        bookmarkText: "URL del marcador",
        permakinkText: "Enlace permanente",
        appInfoText: "GeoExplorer",
        aboutText: "Acerca de GeoExplorer",
        mapInfoText: "Información del mapa",
        descriptionText: "Descripción",
        contactText: "Contacto",
        aboutThisMapText: "Acerca de este mapa"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Mapa",
        tableText: "Table",
        queryText: "Query",
        exportMapText: "Exportar Mapa",
        saveMapText: "Guardar Mapa",
        toolsTitle: "Escoja los elementos que desea incluir en la barra de herramientas:",
        previewText: "Vista previa",
        backText: "Anterior",
        nextText: "Siguiente",
        loginText: "Login",
        loginErrorText: "Invalid username or password.",
        userFieldText: "User",
        passwordFieldText: "Password",
        logoutConfirmTitle: "Warning",
        logoutConfirmMessage: "Logging out will undo any unsaved changes, remove any layers you may have added, and reset the map composition. Do you want to save your composition first?"
    }
});
