/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("de", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div>Zoomstufe: {zoom}</div><div>Maßstab: 1:{scale}</div>",
        loadConfigErrorText: "Gespeicherte Konfiguration kann nicht gelesen werden: <br />",
        loadConfigErrorDefaultText: "Server Fehler.",
        xhrTroubleText: "Kommunikationsfehler: Status ",
        layersText: "Layer",
        titleText: "Titel",
        zoomLevelText: "Zoomstufe",
        bookmarkText: "Bookmark URL",
        permakinkText: 'Permalink',
        appInfoText: "GeoExplorer",
        aboutText: "Über GeoExplorer",
        mapInfoText: "Karteninfo",
        descriptionText: "Beschreibung",
        contactText: "Kontakt",
        aboutThisMapText: "Über diese Karte"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Karte",
        tableText: "Tabelle",
        queryText: "Abfrage",
        exportMapText: "Karte veröffentlichen",
        saveMapText: "Karte speichern",
        toolsTitle: "Werkzeuge, die in der Werkzeugleiste zur Verfügung stehen sollen:",
        previewText: "Vorschau",
        backText: "Zurück",
        nextText: "Weiter",
        loginText: "Login",
        loginErrorText: "Ungültiger Benutzername oder ungültiges Kennwort.",
        userFieldText: "Benutzer",
        passwordFieldText: "Kennwort",
        logoutConfirmTitle: "Warning",
        logoutConfirmMessage: "Logging out will undo any unsaved changes, remove any layers you may have added, and reset the map composition. Do you want to save your composition first?"
    }
});
