Proj4js.maps.EPSG27563 = {
  mapOptions: {
    projection: 'EPSG:27563',
    units: 'm', 
    maxResolution: 2000000/512
  },
  layerName: 'metacarta default',
  layerUrl: 'http://labs.metacarta.com/wms-c/Basic.py',
  layerParams: {'layers':'basic'},
  layerOptions: null
};
