/**
 * Copyright (c) 2008-2010 The Open Source Geospatial Foundation
 *
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

/**
 * @requires GeoExt/Lang.js
 */

GeoExt.Lang.add("fr", {
    "GeoExt.ux.PrintPreview.prototype": {
        paperSizeText: "Taille du papier:",
        resolutionText: "Résolution:",
        printText: "Imprimer",
        emptyTitleText: "Titre",
        emptyCommentText: "Commentaires",
        creatingPdfText: "Creation du PDF ..."
    }
});
