/**
 * Copyright (c) 2009-2010 The Open Planning Project
 */

GeoExt.Lang.add("no", {
    "GeoExplorer.prototype": {
        zoomSliderText: "<div>Zoom nivå: {zoom}</div><div>Målestokk: 1:{scale}</div>",
        loadConfigErrorText: "Problemer med å lese lagret konfigurasjon: <br />",
        loadConfigErrorDefaultText: "Server Feil.",
        xhrTroubleText: "Kommunikasjons Problem: Status ",
        layersText: "Kartlag",
        titleText: "Titel",
        zoomLevelText: "Zoom nivå",
        bookmarkText: "Bokmerk URL",
        permakinkText: 'Permalenke',
        appInfoText: "GeoExplorer",
        aboutText: "Om GeoExplorer",
        mapInfoText: "Kart Info",
        descriptionText: "Beskrivelse",
        contactText: "Kontakt",
        aboutThisMapText: "Om dette kartet"
    },
    "GeoExplorer.Composer.prototype": {
        mapText: "Kart",
        tableText: "Tabell",
        queryText: "Spørring",
        exportMapText: "Publiser kart",
        saveMapText: "Lagre kart",
        toolsTitle: "Velg verktøy og legge til i verktøyslinjen:",
        previewText: "Forhåndsvisning",
        backText: "Tilbake",
        nextText: "Neste",
        loginText: "Login",
        loginErrorText: "Ugylding brukernavn eller passord.",
        userFieldText: "Bruker",
        passwordFieldText: "Passord",
        logoutConfirmTitle: "Advarsel",
        logoutConfirmMessage: "Logge ut vil fjerne siste endringer, fjerne kartlag du har lagt til, og tilbakestille kart komposisjonen. Vil du lagre din komposisjon først?"
   }
});
