/* Translators (2009 onwards):
 *  - Omnipaedista
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["el"]
 * Dictionary for Ελληνικά.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["el"] = OpenLayers.Util.applyDefaults({

    'Scale = 1 : ${scaleDenom}': "Κλίμακα ~ 1 : ${scaleDenom}"

});
