/* Translators (2009 onwards):
 *  - Slomox
 */

/**
 * @requires OpenLayers/Lang.js
 */

/**
 * Namespace: OpenLayers.Lang["nds"]
 * Dictionary for Plattdüütsch.  Keys for entries are used in calls to
 *     <OpenLayers.Lang.translate>.  Entry bodies are normal strings or
 *     strings formatted for use with <OpenLayers.String.format> calls.
 */
OpenLayers.Lang["nds"] = OpenLayers.Util.applyDefaults({

    'unhandledRequest': "Unbehannelt Trüchmellels för de Anfraag ${statusText}",

    'Permalink': "Permalink",

    'Overlays': "Overlays",

    'Base Layer': "Achtergrundkoort",

    'noFID': "En Feature, dat keen FID hett, kann nich aktuell maakt warrn.",

    'browserNotSupported': "Dien Browser ünnerstütt keen Vektorbiller. Ünnerstütt Renderers:\n${renderers}",

    'commitSuccess': "WFS-Transakschoon: hett klappt ${response}",

    'commitFailed': "WFS-Transakschoon: hett nich klappt ${response}",

    'Scale = 1 : ${scaleDenom}': "Skaal = 1 : ${scaleDenom}",

    'methodDeprecated': "Disse Methood is oold un schall dat in 3.0 nich mehr geven. Bruuk dor man beter ${newMethod} för."

});
