/*!
 * Ext JS Library 3.4.0
 * Copyright(c) 2006-2011 Sencha Inc.
 * licensing@sencha.com
 * http://www.sencha.com/license
 */
.x-treegrid-root-table {
    border-right: 1px solid;
}

.x-treegrid-root-node {
    overflow: auto;
}

.x-treegrid-hd-hidden {
    visibility: hidden;
    border: 0;
    width: 0;
}

.x-treegrid-col {
    border-bottom: 1px solid;
    height: 20px;
    overflow: hidden;
    vertical-align: top;
	-o-text-overflow: ellipsis;
	text-overflow: ellipsis;
    white-space: nowrap;
}

.x-treegrid-text {
    padding-left: 4px;
    -moz-user-select: none;
    -khtml-user-select: none;
}

.x-treegrid-resizer {
    border-left:1px solid;
    border-right:1px solid;
    position:absolute;
    left:0;
    top:0;
}

.x-treegrid-header-inner {
    overflow: hidden;
}

.x-treegrid-root-table,
.x-treegrid-col {
    border-color: #ededed;
}

.x-treegrid-resizer {
    border-left-color:#555;
    border-right-color:#555;
}