
GeoExt.Lang.add("el", {

    "gxp.menu.LayerMenu.prototype": {
        layerText: "Επιφάνεια"
    },

    "gxp.plugins.AddLayers.prototype": {
        addActionMenuText: "Προσθέστε επιφάνειες",
        addActionTip: "Προσθέστε επιφάνειες",
        addServerText: "Προσθέστε ένα Νέο Διακομιστή",
        addButtonText: "Προσθέστε επιφάνειες",
        untitledText: "Χωρίς τίτλο",
        addLayerSourceErrorText: "Σφάλμα στην ανάκτηση δυνατοτήτων WMS ({msg}).\nΠαρακαλώ ελέγξτε το url και δοκιμάστε ξανά.",
        availableLayersText: "Διαθέσιμες Επιφάνειες",
        expanderTemplateText: "<p><b>Περίληψη:</b> {abstract}</p>",
        panelTitleText: "Τίτλος",
        layerSelectionText: "Δείτε διαθέσιμα δεδομένα από:",
        doneText: "Ολοκληρώθηκε",
        uploadText: "Ανεβάστε Δεδομένα",
        addFeedActionMenuText: "Add feeds",
        searchText: "Search for layers"
    },
    
    "gxp.plugins.BingSource.prototype": {
        title: "Επιφάνειες Bing",
        roadTitle: "Δρόμοι Bing",
        aerialTitle: "Εναέριες Bing",
        labeledAerialTitle: "Εναέριες Bing με Ετικέτες"
    },

    "gxp.plugins.FeatureEditor.prototype": {
        splitButtonText: "Edit",
        createFeatureActionText: "Create",
        editFeatureActionText: "Modify",
        createFeatureActionTip: "Δημιουργήστε ένα νέο χαρακτηριστικό",
        editFeatureActionTip: "Τροποποιήστε υπάρχον χαρακτηριστικό",
        commitTitle: "Commit message",
        commitText: "Please enter a commit message for this edit:"
    },
    
    "gxp.plugins.FeatureGrid.prototype": {
        displayFeatureText: "Προβολή στον χάρτη",
        firstPageTip: "Πρώτη σελίδα",
        previousPageTip: "Προηγούμενη σελίδα",
        zoomPageExtentTip: "Ζουμ στο εύρος της σελίδας",
        nextPageTip: "Επόμενη σελίδα",
        nextPageTip: "Τελευταία σελίδα",
        totalMsg: "Features {1} to {2} of {0}"
    },

    "gxp.plugins.GoogleEarth.prototype": {
        apiKeyPrompt: "Παρακαλώ εισάγετε το Google API κλειδί για ",
        menuText: "3D Προβολή",
        tooltip: "Αλλάξτε σε 3D Προβολή"
    },
    
    "gxp.plugins.GoogleSource.prototype": {
        title: "Επιφάνειες Google",
        roadmapAbstract: "Προβολή οδικού χάρτη",
        satelliteAbstract: "Προβολή εικόνων από δορυφόρο",
        hybridAbstract: "Προβολή εικόνων με ονόματα οδών",
        terrainAbstract: "Προβολή οδικού χάρτη με ανάγλυφο εδάφους"
    },

    "gxp.plugins.LayerProperties.prototype": {
        menuText: "Επιλογές Επιφάνειας",
        toolTip: "Επιλογές Επιφάνειας"
    },
    
    "gxp.plugins.LayerTree.prototype": {
        rootNodeText: "Επιφάνειες",
        overlayNodeText: "Επιστρώματα",
        baseNodeText: "Υπόβαθρα"
    },

    "gxp.plugins.Legend.prototype": {
        menuText: "Προβολή Υπομνήματος",
        tooltip: "Προβολή Υπομνήματος"
    },

    "gxp.plugins.LoadingIndicator.prototype": {
        loadingMapMessage: "Φόρτωση Χάρτη..."
    },

    "gxp.plugins.MapBoxSource.prototype": {
        title: "MapBox Layers",
        blueMarbleTopoBathyJanTitle: "Blue Marble Topography & Bathymetry (January)",
        blueMarbleTopoBathyJulTitle: "Blue Marble Topography & Bathymetry (July)",
        blueMarbleTopoJanTitle: "Blue Marble Topography (January)",
        blueMarbleTopoJulTitle: "Blue Marble Topography (July)",
        controlRoomTitle: "Control Room",
        geographyClassTitle: "Geography Class",
        naturalEarthHypsoTitle: "Natural Earth Hypsometric",
        naturalEarthHypsoBathyTitle: "Natural Earth Hypsometric & Bathymetry",
        naturalEarth1Title: "Natural Earth I",
        naturalEarth2Title: "Natural Earth II",
        worldDarkTitle: "World Dark",
        worldLightTitle: "World Light",
        worldPrintTitle: "World Print"
    },

    "gxp.plugins.Measure.prototype": {
        buttonText: "Μέτρο",
        lengthMenuText: "Απόσταση",
        areaMenuText: "Εμβαδόν",
        lengthTooltip: "Υπολογίστε απόσταση",
        areaTooltip: "Υπολογίστε εμβαδόν",
        measureTooltip: "Μέτρο"
    },

    "gxp.plugins.Navigation.prototype": {
        menuText: "Μετακινήστε Χάρτη",
        tooltip: "Μετακινήστε Χάρτη"
    },

    "gxp.plugins.NavigationHistory.prototype": {
        previousMenuText: "Ζουμ Στο Προηγούμενο Επίπεδο",
        nextMenuText: "Ζουμ Στο Επόμενο Επίπεδο",
        previousTooltip: "Ζουμ Στο Προηγούμενο Επίπεδο",
        nextTooltip: "Ζουμ Στο Επόμενο Επίπεδο"
    },

    "gxp.plugins.OSMSource.prototype": {
        title: "Επιφάνειες OpenStreetMap",
        mapnikAttribution: "&copy; <a href='http://www.openstreetmap.org/copyright'>OpenStreetMap</a> contributors",
        osmarenderAttribution: "Δεδομένα CC-By-SA by <a href='http://openstreetmap.org/'>OpenStreetMap</a>"
    },

    "gxp.plugins.Print.prototype": {
        buttonText:"Εκτύπωση",
        menuText: "Εκτύπωση Χάρτη",
        tooltip: "Εκτύπωση Χάρτη",
        previewText: "Προεπισκόπηση Εκτύπωσης",
        notAllNotPrintableText: "Δεν Μπορούν Να Εκτυπωθούν Όλες Οι Επιφάνειες",
        nonePrintableText: "Καμία από τις επιφάνειες του τρέχοντος χάρτη δεν μπορεί να εκτυπωθεί"
    },

    "gxp.plugins.MapQuestSource.prototype": {
        title: "Επιφάνειες MapQuest",
        osmAttribution: "Πλακίδια Προσφορά Των <a href='http://open.mapquest.co.uk/' target='_blank'>MapQuest</a> <img src='http://developer.mapquest.com/content/osm/mq_logo.png' border='0'>",
        osmTitle: "MapQuest OpenStreetMap",
        naipAttribution: "Πλακίδια Προσφορά Των of <a href='http://open.mapquest.co.uk/' target='_blank'>MapQuest</a> <img src='http://developer.mapquest.com/content/osm/mq_logo.png' border='0'>",
        naipTitle: "Εικόνες MapQuest"
    },

    "gxp.plugins.QueryForm.prototype": {
        queryActionText: "Επερώτηση",
        queryMenuText: "Επιφάνεια επερώτησης",
        queryActionTip: "Κάντε επερώτηση στην επιλεγμένη επιφάνεια",
        queryByLocationText: "Query by current map extent",
        queryByAttributesText: "Επερώτηση βάσει χαρακτηριστικών",
        queryMsg: "Γίνεται επερώτηση...",
        cancelButtonText: "Ακύρωση",
        noFeaturesTitle: "Κανένα Αποτέλεσμα",
        noFeaturesMessage: "Η επερώτηση σας δεν επέστρεψε αποτελέσματα."
    },

    "gxp.plugins.RemoveLayer.prototype": {
        removeMenuText: "Κατάργηση επιφάνειας",
        removeActionTip: "Κατάργηση επιφάνειας"
    },
    
    "gxp.plugins.Styler.prototype": {
        menuText: "Επεξεργασία Στυλ",
        tooltip: "Επεξεργασία στυλ επιφάνειας"

    },

    "gxp.plugins.WMSGetFeatureInfo.prototype": {
        buttonText:"Identify",
        infoActionTip: "Πάρτε Πληροφορίες Χαρακτηριστικού",
        popupTitle: "Πληροφορίες Χαρακτηριστικού"
    },

    "gxp.plugins.Zoom.prototype": {
        zoomMenuText: "Zoom Box",
        zoomInMenuText: "Ζουμ Μέσα",
        zoomOutMenuText: "Ζουμ Έξω",
        zoomTooltip: "Zoom by dragging a box",
        zoomInTooltip: "Ζουμ Μέσα",
        zoomOutTooltip: "Ζουμ Έξω"
    },
    
    "gxp.plugins.ZoomToExtent.prototype": {
        menuText: "Ζουμ Στο Μέγιστο Εύρος",
        tooltip: "Ζουμ Στο Μέγιστο Εύρος"
    },
    
    "gxp.plugins.ZoomToDataExtent.prototype": {
        menuText: "Ζουμ στο εύρος επιφάνειας",
        tooltip: "Ζουμ στο εύρος επιφάνειας"
    },

    "gxp.plugins.ZoomToLayerExtent.prototype": {
        menuText: "Ζουμ στο εύρος επιφάνειας",
        tooltip: "Ζουμ στο εύρος επιφάνειας"
    },
    
    "gxp.plugins.ZoomToSelectedFeatures.prototype": {
        menuText: "Ζουμ στα συγκεκριμένα χαρακτηριστικά",
        tooltip: "Ζουμ στα συγκεκριμένα χαρακτηριστικά"
    },

    "gxp.FeatureEditPopup.prototype": {
        closeMsgTitle: "Αποθήκευση Αλλαγών;",
        closeMsg: "Αυτό το χαρακτηριστικό έχει τροποποιήσεις που δεν έχουν αποθηκευτεί. Θέλετε να φυλάξετε τις αλλαγές σας;",
        deleteMsgTitle: "Διαγραφή Χαρακτηριστικού",
        deleteMsg: "Είστε σίγουροι οτι θέλετε να διαγράψετε το συγκεκριμένο χαρακτηριστικό",
        editButtonText: "Τροποποίηση",
        editButtonTooltip: "Κάντε αυτό το χαρακτηριστικό τροποποιήσιμο",
        deleteButtonText: "Διαγραφή",
        deleteButtonTooltip: "Διαγράψτε αυτό το χαρακτηριστικό",
        cancelButtonText: "Ακύρωση",
        cancelButtonTooltip: "Σταματήστε την τροποποίηση, απορρίπτοντας οποιεσδήποτε αλλαγές",
        saveButtonText: "Αποθήκευση",
        saveButtonTooltip: "Αποθήκευση αλλαγών"
    },
    
    "gxp.FillSymbolizer.prototype": {
        fillText: "Γέμισμα",
        colorText: "Χρώμα",
        opacityText: "Αδιαφάνεια"
    },
    
    "gxp.FilterBuilder.prototype": {
        builderTypeNames: ["οποιοδήποτε", "όλα", "κανένα", "όχι όλα"],
        preComboText: "Ταύτιση",
        postComboText: "των ακόλουθων:",
        addConditionText: "προσθήκη συνθήκης",
        addGroupText: "προσθήκη ομάδας",
        removeConditionText: "κατάργηση συνθήκης"
    },
    
    "gxp.grid.CapabilitiesGrid.prototype": {
        nameHeaderText : "Όνομα",
        titleHeaderText : "Τίτλος",
        queryableHeaderText : "Μπορούν να γίνουν επερωτήσεις",
        layerSelectionLabel: "Δείτε διαθέσιμα δεδομένα από:",
        layerAdditionLabel: "ή προσθέστε νέο διακομιστή.",
        expanderTemplateText: "<p><b>Περιγραφή:</b> {abstract}</p>"
    },
    
    "gxp.PointSymbolizer.prototype": {
        graphicCircleText: "κύκλος",
        graphicSquareText: "τετράγωνο",
        graphicTriangleText: "τρίγωνο",
        graphicStarText: "αστέρι",
        graphicCrossText: "σταυρός",
        graphicXText: "χ",
        graphicExternalText: "εξωτερικό",
        urlText: "URL",
        opacityText: "αδιαφάνεια",
        symbolText: "Σύμβολο",
        sizeText: "Εύρος",
        rotationText: "Περιστροφή"
    },

    "gxp.QueryPanel.prototype": {
        queryByLocationText: "Επερώτηση βάσει τοποθεσίας",
        currentTextText: "Τρέχον εύρος",
        queryByAttributesText: "Επερώτηση βάσει χαρακτηριστικών",
        layerText: "Επιφάνεια"
    },
    
    "gxp.RulePanel.prototype": {
        scaleSliderTemplate: "{scaleType} Κλίμακα 1:{scale}",
        labelFeaturesText: "Χαρακτηριστικά Επιφάνειας",
        labelsText: "Ετικέτες",
        basicText: "Απλή",
        advancedText: "Προηγμένη",
        limitByScaleText: "Περιορισμός βάσει κλίμακας",
        limitByConditionText: "Περιορισμός βάσει συνθήκης",
        symbolText: "Σύμβολο",
        nameText: "Όνομα"
    },
    
    "gxp.ScaleLimitPanel.prototype": {
        scaleSliderTemplate: "{scaleType} Κλίμακα 1:{scale}",
        minScaleLimitText: "Όριο ελάχιστης κλίμακας",
        maxScaleLimitText: "Όριο μέγιστης κλίμακας"
    },
    
    "gxp.StrokeSymbolizer.prototype": {
        solidStrokeName: "συμπαγής",
        dashStrokeName: "με παύλες",
        dotStrokeName: "με κουκίδες",
        titleText: "Πέρασμα",
        styleText: "Στύλο",
        colorText: "Χρώμα",
        widthText: "Πλάτος",
        opacityText: "Αδιαφάνεια"
    },
    
    "gxp.StylePropertiesDialog.prototype": {   
        titleText: "Γενικά",
        nameFieldText: "Όνομα",
        titleFieldText: "Τίτλος",
        abstractFieldText: "Περίληψη"
    },
    
    "gxp.TextSymbolizer.prototype": {
        labelValuesText: "Τιμές ετικέτας",
        haloText: "Φωτοστέφανο",
        sizeText: "Μέγεθος"
    },
    
    "gxp.WMSLayerPanel.prototype": {
        aboutText: "Σχετικά με",
        titleText: "Τίτλος",
        nameText: "Όνομα",
        descriptionText: "Περιγραφή",
        displayText: "Προβολή",
        opacityText: "Αδιαφάνεια",
        formatText: "Τύπος",
        transparentText: "Διαφανές",
        cacheText: "Cache",
        cacheFieldText: "Χρησιμοποίησε την cached έκδοση",
        stylesText: "Διαθέσιμες Στυλ",
        infoFormatText: "Info format",
        infoFormatEmptyText: "Select a format",
        displayOptionsText: "Display options",
        queryText: "Limit with filters",
        scaleText: "Limit by scale",
        minScaleText: "Min scale",
        maxScaleText: "Max scale",
        switchToFilterBuilderText: "Switch back to filter builder",
        cqlPrefixText: "or ",
        cqlText: "use CQL filter instead",
        singleTileText: "Single tile",
        singleTileFieldText: "Use a single tile"
    },

    "gxp.EmbedMapDialog.prototype": {
        publishMessage: "Ο χάρτης σας είναι έτοιμος να δημοσιευτεί στο διαδίκτυο! Απλά αντιγράψτε τον ακόλουθο HTML κώδικα για να ενσωματώσετε τον χάρτη στην ιστοσελίδα σας:",
        heightLabel: 'Ύψος',
        widthLabel: 'Πλάτος',
        mapSizeLabel: 'Μέγεθος Χάρτη',
        miniSizeLabel: 'Μικρογραφία',
        smallSizeLabel: 'Μικρό',
        premiumSizeLabel: 'Εξαιρετικά ποιοτικό',
        largeSizeLabel: 'Μεγάλος'
    },
    
    "gxp.WMSStylesDialog.prototype": {
         addStyleText: "Προσθήκη",
         addStyleTip: "Προσθήκη νέου στυλ",
         chooseStyleText: "Επιλογή στυλ",
         deleteStyleText: "Αφαίρεση",
         deleteStyleTip: "Διαγραφή του επιλεγμένου στυλ",
         editStyleText: "Επεξεργασία",
         editStyleTip: "Επεξεργασία του επιλεγμένου στυλ",
         duplicateStyleText: "Διπλότυπο",
         duplicateStyleTip: "Φτιάξτε διπλότυπο του επιλεγμένου στύλ",
         addRuleText: "Προσθήκη",
         addRuleTip: "Προσθήκη νέου κανόνα",
         newRuleText: "Νέος Κανόνας",
         deleteRuleText: "Αφαίρεση",
         deleteRuleTip: "Διαγράψτε τον επιλεγμένο κανόνα",
         editRuleText: "Επεξεργασία",
         editRuleTip: "Επεξεργαστείτε τον επιλεγμένο κανόνα",
         duplicateRuleText: "Διπλότυπο",
         duplicateRuleTip: "Φτιάξτε διπλότυπο του επιλεγμένου κανόνα",
         cancelText: "Ακύρωση",
         saveText: "Αποθήκευση",
         styleWindowTitle: "Στυλ Χρήστη: {0}",
         ruleWindowTitle: "Κανόνες Στυλ: {0}",
         stylesFieldsetTitle: "Στυλ",
         rulesFieldsetTitle: "Κανόνες"
    },

    "gxp.LayerUploadPanel.prototype": {
        titleLabel: "Τίτλος",
        titleEmptyText: "Τίτλος επιφάνειας",
        abstractLabel: "Περιγραφή",
        abstractEmptyText: "Περιγραφή επιφάνειας",
        fileLabel: "Δεδομένα",
        fieldEmptyText: "Βρείτε το μονοπάτι για ένα αρχείο δεδομένων...",
        uploadText: "Ανέβασμα",
        uploadFailedText: "Upload failed",
        processingUploadText: "Processing upload...",
        waitMsgText: "Τα δεδομένα σας ανεβαίνουν...",
        invalidFileExtensionText: "Ο τύπος αρχείου πρέπει να είναι ένας από τους: ",
        optionsText: "Επιλογές",
        workspaceLabel: "Χώρος εργασίας",
        workspaceEmptyText: "Προεπιλεγμένος χώρος εργασίας",
        dataStoreLabel: "Αποθήκη",
        dataStoreEmptyText: "Create new store",
        defaultDataStoreEmptyText: "Προεπιλεγμένη αποθήκη"
    },
    
    "gxp.NewSourceWindow.prototype": {
        title: "Προσθήκη Νέου Διακομιστή...",
        cancelText: "Ακύρωση",
        addServerText: "Προσθήκη Διακομιστή",
        invalidURLText: "Είσαγετε ένα έγκυρο URL για ένα WMS endpoint (e.g. http://example.com/geoserver/wms)",
        contactingServerText: "Επικοινωνία με Διακομιστή..."
    },

    "gxp.ScaleOverlay.prototype": { 
        zoomLevelText: "Επίπεδο ζουμ"
    },

    "gxp.Viewer.prototype": {
        saveErrorText: "Trouble saving: "
    },

    "gxp.FeedSourceDialog.prototype": {
        feedTypeText: "Πηγή",
        addPicasaText: "Φωτογραφίες Picasa",
        addYouTubeText: "YouTube βίντεο",
        addRSSText: "Άλλα Feed GeoRSS",
        addFeedText: "Προσθήκη στο χάρτη",
        addTitleText: "Τίτλος",
        keywordText: "Λέξη-κλειδί",
        doneText: "Done",
        titleText: "Προσθήκη ροής",
        maxResultsText: "Max Είδη"
    }

});
