/**
 * Copyright (c) 2008-2012 The Open Source Geospatial Foundation
 *
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

/**
 * @requires GeoExt/Lang.js
 */

GeoExt.Lang.add("fr", {
    "GeoExt.tree.LayerContainer.prototype": {
        text: "Couches"
    },
    "GeoExt.tree.BaseLayerContainer.prototype": {
        text: "Couches de base"
    },
    "GeoExt.tree.OverlayLayerContainer.prototype": {
        text: "Couches additionnelles"
    }
});
