/**
 * Copyright (c) 2008-2011 The Open Source Geospatial Foundation
 *
 * Published under the BSD license.
 * See http://svn.geoext.org/core/trunk/geoext/license.txt for the full text
 * of the license.
 */

/**
 * @requires GeoExt/Lang.js
 */

GeoExt.Lang.add("es", {

    "GeoExt.ux.PrintPreview.prototype": {   
        paperSizeText: "Tamaño:",
        resolutionText: "Resolución:",
        printText: "Imprimir",
        emptyTitleText: "Escriba el título del mapa aquí.",
        includeLegendText: "¿Incluir leyenda?",
        emptyCommentText: "Escriba aquí sus comentarios.",
        creatingPdfText: "Creando PDF..."
    }
    
});
