/**
 *  Test.AnotherWay version 0.5
 *  
 *  Copyright (c) 2005 Artem Khodush, http://straytree.org
 *  
 *  Permission is hereby granted, free of charge, to any person obtaining
 *  a copy of this software and associated documentation files (the
 *  "Software"), to deal in the Software without restriction, including
 *  without limitation the rights to use, copy, modify, merge, publish,
 *  distribute, sublicense, and/or sell copies of the Software, and to
 *  permit persons to whom the Software is furnished to do so, subject to
 *  the following conditions:
 *  
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *  
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *  NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *  
 */

* {
    padding: 0;
    margin: 0;
}

html {
    height: 99%;
}

body {
    height: 98%;
    font: normal normal 10pt sans-serif
}

#col1 {
    float: left;
    width: 27em;
    margin: 0 0 0 1em;
    overflow: visible;
}

#col2 {
    position: relative;
    height: 98%;
    margin: 0 0.5em 0 28em;
}

#col1_header {
    margin-top: 0.5em;
}

#scroller {
    height: 400px;
    overflow: auto;
}

#testtable {
    margin: 0 0 2em 0;
    width: 97%;
    font-size: 1em;
    border-collapse: collapse;
}
#testtable input {
    cursor: pointer;
}
#testtable td {
    line-height: 2em;
    padding: 0;
    margin: 0;
    border-top: 1px solid #ccc;
    border-bottom: 1px solid #ccc;
}
#testtable tr:hover td {
    background-color: #ededed;
}
#testtable tr.isHidden {
    display: none;
}
#testtable tr.isShown {
    display: table-row;
}

#run_buttons, #running-time {
    margin-bottom: 1em;
}

#right_header {
    padding-top: 0.8em;
}

#results_count {
    float: left;
}

#results > p:hover {
    background-color: #ededed;
}

.active_tab {
    float: right;
    padding: 0 1em 0.2em 1em;
    background: #0af;
    border: 1px solid #048;
    border-bottom: none;
    cursor: pointer;
    cursor: hand;
    position: relative;
    top: -0.2em;
}

.inactive_tab {
    float: right;
    padding: 0 1em 0 1em;
    background: #9bb;
    color: #444;
    border: 1px solid #9bb;
    border-bottom: none;
    cursor: pointer;
    cursor: hand;
}

.inactive_mouseover_tab {
    float: right;
    padding: 0 1em 0 1em;
    background: #9bb;
    color: #062;
    border: 1px solid #062;
    border-bottom: none;
    cursor: pointer;
    cursor: hand;
}

#right_frame {
    overflow: auto;
    position: relative;
    top: -0.2em;
    clear: right;
    height: 95%;
    border: 1px solid #048;
}

#debug {
    display: none;
}

#debug p {
    margin: 2px 0 0 5em;
    text-indent: -4.8em;
}

#error {
    display: none;
    color: #c22;
}

#results p {
    margin: 0 0 2px 0;
}

/* cursor indicating that detailed results may be expanded/contracted */
#results p.badtest {
    cursor: text;
}

#results p.ok, #results p.fail {
    cursor: pointer;
    cursor: hand;
}

/* colored squares in the results window at the left of test page names */
#results p.ok .bullet {
    background: #6d6;
}

#results p.fail .bullet {
    background: #d46;
}

#results p.badtest .bullet {
    background: #ea3;
}

#results p.loading .bullet {
    background: #48f;
}

#results p.running .bullet {
    background: #26e;
}

#results p.waiting .bullet {
    background: #04d;
}

/* highlight in the results line */
#results p .warning {
    background: #ffc;
}

/* layout of the detailed results */
.result_detail {
    padding-left: 3em;
}

.result_exception_detail {
    padding-left: 4em;
}

.result_exception_stack_detail {
    padding-left: 5em;
}

.result_micro_detail {
    padding-left: 6em;
}

/* colouring in the detailed results */
.result_detail .fail, .result_exception_detail .fail, .result_micro_detail .fail {
    background: #ffd8d8;
}

/* "start recording" controls*/
#record_div {
    margin-top: 3em;
}

#record_div p {
    margin-bottom: 0.5em;
}

#record_select {
    width: 88%;
}

#record_input {
    width: 53%;
}